package imm.se.vending.model;

import java.util.HashMap;
import java.util.Map;
import java.util.Observable;

public class VendingMachine extends Observable {

	public static final VendingMachineState IDLE_STATE = new IdleState();
	public static final VendingMachineState BANANA_SELECTED_STATE = new FruitSelectedState(Fruit.BANANA);
	public static final VendingMachineState APPLE_SELECTED_STATE = new FruitSelectedState(Fruit.APPLE);

	private Map<Fruit, Integer> fruits = new HashMap<Fruit, Integer>(); // public
																		// attribute
																		// read-only
																		// for
																		// public
	private Map<Fruit, Integer> prices = new HashMap<Fruit, Integer>(); // public
																		// attribute
																		// read-only
																		// for
																		// public
	private int currentMoney = 0; // public attribute read-only for public
	private int totalMoney = 0; // public attribute read-only for public
	private int rest = 0; // public attribute read-only for public
	private Fruit dispensedItem = null; // public attribute read-only for public

	private VendingMachineState state = new IdleState(); // package visible
															// attribute
	public VendingMachine(int b, int a) {
		prices.put(Fruit.APPLE, new Integer(3));
		prices.put(Fruit.BANANA, new Integer(7));
		setFruit(Fruit.BANANA, b);
		setFruit(Fruit.APPLE, a);
	}

	public int getNumberOfFruit(Fruit fruit) {
		if (fruit == null) {
			return 0;
		}
		return fruits.get(fruit);
	}

	public void input(int i) {
		clearDispenser();
		state.input(this, i);
	}

	public void selectFruit(Fruit fruit) {
		clearDispenser();
		state.selectFruit(this, fruit);
	}

	public void cancel() {
		clearDispenser();
		state.cancel(this);
	}

	public int getCurrentMoney() {
		return currentMoney;
	}

	public int getTotalMoney() {
		return totalMoney;
	}

	public int getRest() {
		return rest;
	}

	public Fruit selectedFruit() {
		return state.selectedFruit();
	}

	public Fruit getDispensedItem() {
		return dispensedItem;
	}

	public int getPrice(Fruit fruit) {
		if (fruit == null) {
			return Integer.MAX_VALUE;
		}
		return ((Integer) prices.get(fruit)).intValue();
	}

	void setIdleState() {
		setCurrentState(IDLE_STATE);
	}

	void setCurrentStateForFruit(Fruit fruit) {
		switch (fruit) {
		case APPLE:
			setCurrentState(APPLE_SELECTED_STATE);
			break;
		case BANANA:
			setCurrentState(BANANA_SELECTED_STATE);
			break;
		}
	}

	void dispense(Fruit fruit) {
		if (fruit == null) {
			dispense(null, currentMoney);
		} else {
			setFruit(fruit, getNumberOfFruit(fruit) - 1);
			int price = getPrice(fruit);
			incrementTotalMoney(price);
			dispense(fruit, currentMoney - price);
		}
	}

	void dispense(Fruit fruit, int rest) {
		setRest(rest);
		setCurrentMoney(0);
		setDispensedItem(fruit);
	}

	void clearDispenser() {
		setRest(0);
		setDispensedItem(null);
	}

	void incrementTotalMoney(int price) {
		totalMoney += price;
		setChanged();
		notifyObservers(NotificationType.TOTAL_MONEY);
	}

	boolean hasEnoughMoneyFor(Fruit fruit) {
		return getCurrentMoney() >= getPrice(fruit);
	}

	public boolean hasFruit(Fruit fruit) {
		return getNumberOfFruit(fruit) > 0;
	}

	void setRest(int i) {
		rest = i;
		setChanged();
		notifyObservers(NotificationType.REST);
	}

	void setFruit(Fruit fruit, int value) {
		fruits.put(fruit, value);
		setChanged();
		notifyObservers(NotificationType.FRUIT);
	}

	void setCurrentMoney(int i) {
		currentMoney = i;
		setChanged();
		notifyObservers(NotificationType.CURRENT_MONEY);
	}

	void setCurrentState(VendingMachineState s) {
		state = s;
		setChanged();
		notifyObservers(NotificationType.CURRENT_STATE);
	}

	void setDispensedItem(Fruit dispensedItem) {
		this.dispensedItem = dispensedItem;
		setChanged();
		notifyObservers(NotificationType.DISPENSED_ITEM);
	}

}
